#include "StdAfx.h"
#include "EngineVR.h"

HDC    hDC; // Private Device Context
HGLRC  hRC; // Rendering Context Permater

EngineVR::EngineVR(void)
{
	SceneList    = NULL;
    SceneListLen = 0;
    CurrentScene = -1;

	hDC = NULL;
    hRC = NULL;
}


EngineVR::~EngineVR(void)
{
	SceneList[CurrentScene].Draw();
}

void EngineVR::InitEngineVR()
{
	glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glClearColor(0.0f,0.0f,0.0f,1.0f);
    
	glClearStencil(0);
    
	// Perspective correction setting
    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
    
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);  	 // Set Culling Face To Back Face
    
	//glDisable (GL_COLOR_MATERIAL);
    glShadeModel(GL_FLAT);
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE); // Normals normalization
    
	glLineWidth(1);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

void EngineVR::CreateSet(unsigned int NumberOfScenes)
{
	SceneList    = new TScene[NumberOfScenes];
    SceneListLen = NumberOfScenes;
    CurrentScene = 0;
}

void EngineVR::DestroySet()
{
	int i;
	for(i=0; i<SceneListLen; i++) SceneList[i].DestroyScene();
	delete [] SceneList;
    SceneListLen = 0;
    CurrentScene = -1;
}


int EngineVR::CreateSceneView(HWND Wnd,
	                          int Width, 
							  int Height,
							  int DepthBufferBits, 
							  int ColorBufferBits)
{
	   int PixelFormat;
    
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,                  // Version Number
		PFD_DRAW_TO_WINDOW| // supported format by the window
		PFD_SUPPORT_OPENGL| // OpenGL capable
		PFD_DOUBLEBUFFER,   // enabling the double buffering
		PFD_TYPE_RGBA,      // RGBA color format
		ColorBufferBits,    // number of bits per color
		0,0,0,0,0,0,        // ignore
		0,                  // No alpha buffer
		0,                  // shift bit (ignore)
		0,                  // no accumulation buffer
		0,0,0,0,            // accumulation bits (ignore)
		DepthBufferBits,    // 16 bit Z-Buffer (Depth buffer)
		1,                  // Stencil Buffer
		0,                  // No auxiliary Buffer
		PFD_MAIN_PLANE,     // main drawing layer
		0,                  // riserved
		0,0,0               // layer Mask (ignore)
	};
	// is the DC available?
	if(!(hDC=GetDC(Wnd))) {  KillSceneView();  return FALSE;}

	// pecifies pixelformat
	PixelFormat = ChoosePixelFormat(hDC, &pfd);
    if(!SetPixelFormat(hDC,PixelFormat,&pfd))      { KillSceneView();  return FALSE;}

	// is it possible to obtain a rendering context?
    if(!(hRC=wglCreateContext(hDC)))               { KillSceneView();  return FALSE;}

    // rendering context activation
    if(!wglMakeCurrent(hDC,hRC))                   { KillSceneView();  return FALSE;}

    ResizeSceneView(Width,Height); // sizes the rendering view

    return TRUE;
}

void EngineVR::ResizeSceneView(unsigned int width, unsigned int height)
{
	if(height==0) height=1;
	ViewPortWidth  = width;
	ViewPortHeight = height;
	glViewport(0,0,(GLint)width,(GLint)height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.1f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void EngineVR::KillSceneView()
{
    if(hRC)// se esiste il rendering context
	{
		/*ReleaseDC(hWnd,hDC);
		wglMakeCurrent(NULL,NULL);
		wglDeleteContext(hRC);
		hRC=NULL;*/
	}
}

void EngineVR::ClearBuffers(int cb, int db)
{
	
	GLbitfield mask = 0;
    if(cb) mask  = GL_COLOR_BUFFER_BIT;
    if(db) mask |= GL_DEPTH_BUFFER_BIT;

	glClearColor(SceneList[CurrentScene].ClearColor.r,
		         SceneList[CurrentScene].ClearColor.g,
				 SceneList[CurrentScene].ClearColor.b,
				 SceneList[CurrentScene].ClearColor.a);

    glClear(mask);
}

void EngineVR::ClearBuffers()
{
	ClearBuffers(1,1);
}


void EngineVR::RenderScene()
{
	if(SceneList && CurrentScene>=0)
	   SceneList[CurrentScene].Draw();
}

void EngineVR::RenderScene(unsigned int Scene)
{
	CurrentScene = Scene;
	if(SceneList && CurrentScene>=0)
	   SceneList[CurrentScene].Draw();
}

void EngineVR::RenderToTexture(TTexture * t)
{
	if(t && t->Texture)
	{
		glViewport(0,0,t->Width,t->Height);
		ClearBuffers();
        RenderScene();
		glBindTexture(GL_TEXTURE_2D,t->Texture);
		glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,t->Width,t->Height);
		glViewport(0,0,ViewPortWidth,ViewPortHeight);
	}
}

void EngineVR::FlipBuffers()
{
	glFlush();
    SwapBuffers(hDC);
}

void EngineVR::BuildSceneMatrices()
{
	SceneList[CurrentScene].BuildSceneMatrices();
}

void EngineVR::BuildSceneGeometry()
{
	SceneList[CurrentScene].BuildSceneGeometry();
}