#include "StdAfx.h"
#include "TLight.h"


TLight::TLight(void)
{
	SetColor(Specular,1.0f,1.0f,1.0f,1.0f);
	SetColor(Diffuse,0.3f,0.3f,0.3f,1.0f);
	SetColor(Ambient,0.1f,0.1f,0.1f,1.0f);
	SetPosition(0,0,0);
	SetDirection(0,-1,0);
	Cutoff = 45;
	SpotProfile = 0.5;
	ConstantAttenuation = 0;
	LinearAttenuation = 0;
	QuadraticAttenuation = 0;
	T.LoadIdentity();
	Enabled  = 1;
	Index = 0;
}


TLight::~TLight(void)
{
}

void TLight::EnableLight(int enable)
{
	if(Index>=0 && Index<8)
	{
		if(enable) glEnable (GL_LIGHT0+Index);
		else       glDisable(GL_LIGHT0+Index);
	}
	Enabled = enable;
}

void TLight::BuildMatrix()
{
	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(Position[0],Position[1],Position[2]);
    glLightfv(GL_LIGHT0+Index,GL_POSITION,Position);
    glLightfv(GL_LIGHT0+Index,GL_SPOT_DIRECTION,Direction);
	glGetFloatv( GL_MODELVIEW_MATRIX , T.m);
}

void TLight::SetPosition(float x, float y, float z)
{
	Position[0] = x;
	Position[1] = y;
	Position[2] = z;
	glLightfv(GL_LIGHT0+Index,GL_POSITION,Position);
}

void TLight::SetDirection(float x, float y, float z)
{
	Direction[0] = x;
	Direction[1] = y;
	Direction[2] = z;
	glLightfv(GL_LIGHT0+Index,GL_SPOT_DIRECTION,Direction);
}

void TLight::LookAt(float x, float y, float z)
{
	TVector v1,v2;
	v1.vector(Position[0],
		      Position[1],
			  Position[2]);
	v2.vector(x,y,z);
	v2-=v1;
	v2.Normalize();
	Direction[0] = v2.x;
	Direction[1] = v2.y;
	Direction[2] = v2.z;
	glLightfv(GL_LIGHT0+Index,GL_SPOT_DIRECTION,Direction);
}

void TLight::LoadLight()
{
	glEnable (GL_LIGHT0+Index);
	glLightfv(GL_LIGHT0+Index,GL_AMBIENT              , Ambient);
	glLightfv(GL_LIGHT0+Index,GL_DIFFUSE              , Diffuse);
	glLightfv(GL_LIGHT0+Index,GL_SPECULAR             , Specular);
	/*glLightfv(GL_LIGHT0+Index,GL_POSITION             , SceneList[SelectedScene].LightList[LIndex].Position);
	glLightfv(GL_LIGHT0+Index,GL_SPOT_DIRECTION       , SceneList[SelectedScene].LightList[LIndex].Direction);*/
	glLightfv(GL_LIGHT0+Index,GL_SPOT_CUTOFF          ,&Cutoff);
	glLightfv(GL_LIGHT0+Index,GL_SPOT_EXPONENT        ,&SpotProfile);
	glLightfv(GL_LIGHT0+Index,GL_CONSTANT_ATTENUATION ,&ConstantAttenuation);
    glLightfv(GL_LIGHT0+Index,GL_LINEAR_ATTENUATION   ,&LinearAttenuation);
    glLightfv(GL_LIGHT0+Index,GL_QUADRATIC_ATTENUATION,&QuadraticAttenuation);
}