#include "StdAfx.h"
#include "TMaterial.h"

#define SetColor(c,r,g,b,a)  c[0]=r,c[1]=g,c[2]=b,c[3]=a
#define SetPosition(p,x,y,z) p[0]=x,p[1]=y,p[2]=z
#define SetDirection(p,x,y,z) p[0]=x,p[1]=y,p[2]=z

TMaterial::TMaterial(void)
{
	LoadDefaultMaterial();
}


TMaterial::~TMaterial(void)
{
}

void TMaterial::LoadDefaultMaterial()
{
	SetColor(Specular,1.0f,1.0f,1.0f,1.0f);
	SetColor(Diffuse ,0.9f,0.9f,0.9f,1.0f);
	SetColor(Ambient ,0.7f,0.7f,0.7f,1.0f);
	SetColor(Emissive,0.0f,0.0f,0.0f,1.0f);
	SetColor(Color   ,0.5f,0.5f,0.5f,1.0f);
	Shininess   = 40.0f;
	Smoothing   = 0;
	Visible     = 1;
	Doublesided = 0;
	Wireframe   = 0;
	TextureLevelList = NULL;
    TextureLevelListLen = 0;
}

void TMaterial::LoadMaterial()
{
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT  ,Ambient   ); 
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE  ,Diffuse   );
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR ,Specular  );
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION ,Emissive  );
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS,&Shininess);
	
	glColor4f(Color[0],Color[1],Color[2],Color[3]);
	
	if(Doublesided) glDisable(GL_CULL_FACE);
	else            glEnable(GL_CULL_FACE);
	
	if(Wireframe)   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	else            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	
	if(Smoothing)   glShadeModel(GL_SMOOTH);
	else            glShadeModel(GL_FLAT);
}

int TMaterial::AddTextureLevel(unsigned int TexIndex, 
	                           unsigned int Operation)
{
	TTextureLevel * NewTextureLevelList = new TTextureLevel[TextureLevelListLen+1];
	memcpy(NewTextureLevelList,TextureLevelList,sizeof(TTextureLevel)*TextureLevelListLen);
	NewTextureLevelList[TextureLevelListLen].TextureIndex = TexIndex;
	NewTextureLevelList[TextureLevelListLen].EnvMode      = TX_MODE_REPLACE;
	NewTextureLevelList[TextureLevelListLen].CombineRGB   = TX_RGB_REPLACE;
	NewTextureLevelList[TextureLevelListLen].CombineAlpha = TX_ALPHA_REPLACE;
	NewTextureLevelList[TextureLevelListLen].Visible      = 1;
	NewTextureLevelList[TextureLevelListLen].Blend        = 0;
	NewTextureLevelList[TextureLevelListLen].BlendSrc     = TX_BLEND_ONE;
	NewTextureLevelList[TextureLevelListLen].BlendDst     = TX_BLEND_ZERO;
	NewTextureLevelList[TextureLevelListLen].MappingType  = MAPPING_UV;
	delete [] TextureLevelList;
	TextureLevelList = NewTextureLevelList;
    TextureLevelListLen++;
	return TextureLevelListLen;
}

int TMaterial::RemoveTextureLevel(unsigned int Level)
{
	if(Level>=0 && Level<TextureLevelListLen)
	{
		TTextureLevel * NewTextureLevelList = new TTextureLevel[TextureLevelListLen-1];
		if(Level == 0)
		{
			memcpy(NewTextureLevelList,&TextureLevelList[1],sizeof(TTextureLevel)*(TextureLevelListLen-1));
		}
		else
		if(Level == TextureLevelListLen-1)
		{
			memcpy(NewTextureLevelList,TextureLevelList,sizeof(TTextureLevel)*(TextureLevelListLen-1));
		}
		else
		{
			memcpy(NewTextureLevelList,TextureLevelList,sizeof(TTextureLevel)*Level);
			memcpy(&NewTextureLevelList[Level],&TextureLevelList[Level+1],sizeof(TTextureLevel)*(TextureLevelListLen-Level-1));
		}
	}
	return TextureLevelListLen;
}
