#include "StdAfx.h"
#include "TMatrix4x4.h"

void  TMatrix4x4::Clear()
{
	m[ 0]=m[ 1]=m[ 2]=m[ 3]=
	m[ 4]=m[ 5]=m[ 6]=m[ 7]=
	m[ 8]=m[ 9]=m[10]=m[11]=
	m[12]=m[13]=m[14]=m[15]=0.0f;
}

void  TMatrix4x4::LoadIdentity()
{
	m[ 1]=m[ 2]=m[ 3]=m[ 4]=
    m[ 6]=m[ 7]=m[ 8]=m[ 9]=
    m[11]=m[12]=m[13]=m[14]=0.0f;
    m[ 0]=m[ 5]=m[10]=m[15]=1.0f;
}

void  TMatrix4x4::Transpose()
{
	float a;
	a=m[ 1]; m[ 1]=m[ 4]; m[ 4]=a;
	a=m[ 2]; m[ 2]=m[ 8]; m[ 8]=a;
	a=m[ 3]; m[ 3]=m[12]; m[12]=a;
	a=m[ 6]; m[ 6]=m[ 9]; m[ 9]=a;
	a=m[ 7]; m[ 7]=m[13]; m[13]=a;
	a=m[11]; m[11]=m[14]; m[14]=a;
}

float TMatrix4x4::Determinant()
{
	TMatrix4x4 result;
	float d12 = (m[ 8] * m[13] - m[12] * m[ 9]);
	float d13 = (m[ 8] * m[14] - m[12] * m[10]);
	float d23 = (m[ 9] * m[14] - m[13] * m[10]);
	float d24 = (m[ 9] * m[15] - m[13] * m[11]);
	float d34 = (m[10] * m[15] - m[14] * m[11]);
	float d41 = (m[11] * m[12] - m[15] * m[ 8]);
	float tmp[16];
	tmp[0] =  (m[5] * d34 - m[6] * d24 + m[7] * d23);
	tmp[1] = -(m[4] * d34 + m[6] * d41 + m[7] * d13);
	tmp[2] =  (m[4] * d24 + m[5] * d41 + m[7] * d12);
	tmp[3] = -(m[4] * d23 - m[5] * d13 + m[6] * d12);
	float det = m[0] * tmp[0] + m[1] * tmp[1] + m[2] * tmp[2] + m[3] * tmp[3];
	return det;
}

void  TMatrix4x4::Invert()
{
	TMatrix4x4 result;
	float d12 = (m[ 8] * m[13] - m[12] * m[ 9]);
	float d13 = (m[ 8] * m[14] - m[12] * m[10]);
	float d23 = (m[ 9] * m[14] - m[13] * m[10]);
	float d24 = (m[ 9] * m[15] - m[13] * m[11]);
	float d34 = (m[10] * m[15] - m[14] * m[11]);
	float d41 = (m[11] * m[12] - m[15] * m[ 8]);
	float tmp[16];
	tmp[0] =  (m[5] * d34 - m[6] * d24 + m[7] * d23);
	tmp[1] = -(m[4] * d34 + m[6] * d41 + m[7] * d13);
	tmp[2] =  (m[4] * d24 + m[5] * d41 + m[7] * d12);
	tmp[3] = -(m[4] * d23 - m[5] * d13 + m[6] * d12);
	float det = m[0] * tmp[0] + m[1] * tmp[1] + m[2] * tmp[2] + m[3] * tmp[3];
	if( det == 0.0 )
	{
		result.LoadIdentity();
	}
    else
	{  float invDet = 1.0f / det;
	   tmp[0] *= invDet;
	   tmp[1] *= invDet;
	   tmp[2] *= invDet;
	   tmp[3] *= invDet;
	   tmp[4] = -(m[1] * d34 - m[2] * d24 + m[3] * d23) * invDet;
	   tmp[5] =  (m[0] * d34 + m[2] * d41 + m[3] * d13) * invDet;
	   tmp[6] = -(m[0] * d24 + m[1] * d41 + m[3] * d12) * invDet;
	   tmp[7] =  (m[0] * d23 - m[1] * d13 + m[2] * d12) * invDet;
	   d12 = m[0] * m[5] - m[4] * m[1];
	   d13 = m[0] * m[6] - m[4] * m[2];
	   d23 = m[1] * m[6] - m[5] * m[2];
	   d24 = m[1] * m[7] - m[5] * m[3];
	   d34 = m[2] * m[7] - m[6] * m[3];
	   d41 = m[3] * m[4] - m[7] * m[0];
	   tmp[8]  =  (m[13] * d34 - m[14] * d24 + m[15] * d23) * invDet;
	   tmp[9]  = -(m[12] * d34 + m[14] * d41 + m[15] * d13) * invDet;
	   tmp[10] =  (m[12] * d24 + m[13] * d41 + m[15] * d12) * invDet;
	   tmp[11] = -(m[12] * d23 - m[13] * d13 + m[14] * d12) * invDet;
	   tmp[12] = -(m[ 9] * d34 - m[10] * d24 + m[11] * d23) * invDet;
	   tmp[13] =  (m[ 8] * d34 + m[10] * d41 + m[11] * d13) * invDet;
	   tmp[14] = -(m[ 8] * d24 + m[ 9] * d41 + m[11] * d12) * invDet;
	   tmp[15] =  (m[ 8] * d23 - m[ 9] * d13 + m[10] * d12) * invDet;
	   memcpy( result.m, tmp, 16*sizeof(float) );
	}
    *this = result;
}

void  TMatrix4x4::Mul(TMatrix4x4 * M)
{
	TMatrix4x4 r;
	r.m[ 0] = m[ 0]*M->m[ 0] + m[ 1]*M->m[ 4] + m[ 2]*M->m[ 8] + m[ 3]*M->m[12];
	r.m[ 1] = m[ 0]*M->m[ 1] + m[ 1]*M->m[ 5] + m[ 2]*M->m[ 9] + m[ 3]*M->m[13];
	r.m[ 2] = m[ 0]*M->m[ 2] + m[ 1]*M->m[ 6] + m[ 2]*M->m[10] + m[ 3]*M->m[14];
	r.m[ 3] = m[ 0]*M->m[ 3] + m[ 1]*M->m[ 7] + m[ 2]*M->m[11] + m[ 3]*M->m[15];
	r.m[ 4] = m[ 4]*M->m[ 0] + m[ 5]*M->m[ 4] + m[ 6]*M->m[ 8] + m[ 7]*M->m[12];
	r.m[ 5] = m[ 4]*M->m[ 1] + m[ 5]*M->m[ 5] + m[ 6]*M->m[ 9] + m[ 7]*M->m[13];
	r.m[ 6] = m[ 4]*M->m[ 2] + m[ 5]*M->m[ 6] + m[ 6]*M->m[10] + m[ 7]*M->m[14];
	r.m[ 7] = m[ 4]*M->m[ 3] + m[ 5]*M->m[ 7] + m[ 6]*M->m[11] + m[ 7]*M->m[15];
	r.m[ 8] = m[ 8]*M->m[ 0] + m[ 9]*M->m[ 4] + m[10]*M->m[ 8] + m[11]*M->m[12];
	r.m[ 9] = m[ 8]*M->m[ 1] + m[ 9]*M->m[ 5] + m[10]*M->m[ 9] + m[11]*M->m[13];
	r.m[10] = m[ 8]*M->m[ 2] + m[ 9]*M->m[ 6] + m[10]*M->m[10] + m[11]*M->m[14];
	r.m[11] = m[ 8]*M->m[ 3] + m[ 9]*M->m[ 7] + m[10]*M->m[11] + m[11]*M->m[15];
	r.m[12] = m[12]*M->m[ 0] + m[13]*M->m[ 4] + m[14]*M->m[ 8] + m[15]*M->m[12];
	r.m[13] = m[12]*M->m[ 1] + m[13]*M->m[ 5] + m[14]*M->m[ 9] + m[15]*M->m[13];
	r.m[14] = m[12]*M->m[ 2] + m[13]*M->m[ 6] + m[14]*M->m[10] + m[15]*M->m[14];
	r.m[15] = m[12]*M->m[ 3] + m[13]*M->m[ 7] + m[14]*M->m[11] + m[15]*M->m[15];
    *this = r;
}

void  TMatrix4x4::Scalar(float s)
{
}

TVector TMatrix4x4::operator * (TVector & v)
{
	TVector V;
	V.x=m[ 0]*v.x + m[ 1]*v.y + m[ 2]*v.z + m[ 3];
	V.y=m[ 4]*v.x + m[ 5]*v.y + m[ 6]*v.z + m[ 7];
	V.z=m[ 8]*v.x + m[ 9]*v.y + m[10]*v.z + m[11];
	return V;
}

TMatrix4x4 TMatrix4x4::operator * (TMatrix4x4 & M)
{
	TMatrix4x4 r;
	r.m[ 0] = m[ 0]*M.m[ 0] + m[ 1]*M.m[ 4] + m[ 2]*M.m[ 8] + m[ 3]*M.m[12];
	r.m[ 1] = m[ 0]*M.m[ 1] + m[ 1]*M.m[ 5] + m[ 2]*M.m[ 9] + m[ 3]*M.m[13];
	r.m[ 2] = m[ 0]*M.m[ 2] + m[ 1]*M.m[ 6] + m[ 2]*M.m[10] + m[ 3]*M.m[14];
	r.m[ 3] = m[ 0]*M.m[ 3] + m[ 1]*M.m[ 7] + m[ 2]*M.m[11] + m[ 3]*M.m[15];
	r.m[ 4] = m[ 4]*M.m[ 0] + m[ 5]*M.m[ 4] + m[ 6]*M.m[ 8] + m[ 7]*M.m[12];
	r.m[ 5] = m[ 4]*M.m[ 1] + m[ 5]*M.m[ 5] + m[ 6]*M.m[ 9] + m[ 7]*M.m[13];
	r.m[ 6] = m[ 4]*M.m[ 2] + m[ 5]*M.m[ 6] + m[ 6]*M.m[10] + m[ 7]*M.m[14];
	r.m[ 7] = m[ 4]*M.m[ 3] + m[ 5]*M.m[ 7] + m[ 6]*M.m[11] + m[ 7]*M.m[15];
	r.m[ 8] = m[ 8]*M.m[ 0] + m[ 9]*M.m[ 4] + m[10]*M.m[ 8] + m[11]*M.m[12];
	r.m[ 9] = m[ 8]*M.m[ 1] + m[ 9]*M.m[ 5] + m[10]*M.m[ 9] + m[11]*M.m[13];
	r.m[10] = m[ 8]*M.m[ 2] + m[ 9]*M.m[ 6] + m[10]*M.m[10] + m[11]*M.m[14];
	r.m[11] = m[ 8]*M.m[ 3] + m[ 9]*M.m[ 7] + m[10]*M.m[11] + m[11]*M.m[15];
	r.m[12] = m[12]*M.m[ 0] + m[13]*M.m[ 4] + m[14]*M.m[ 8] + m[15]*M.m[12];
	r.m[13] = m[12]*M.m[ 1] + m[13]*M.m[ 5] + m[14]*M.m[ 9] + m[15]*M.m[13];
	r.m[14] = m[12]*M.m[ 2] + m[13]*M.m[ 6] + m[14]*M.m[10] + m[15]*M.m[14];
	r.m[15] = m[12]*M.m[ 3] + m[13]*M.m[ 7] + m[14]*M.m[11] + m[15]*M.m[15];
    return r;
}