#include "StdAfx.h"
#include "TObject.h"


TObject::TObject(void)
{
	Pose.Clear();
	Scale.vector(1.0f,1.0f,1.0f);
	TRS.LoadIdentity();
	SurfaceList     = NULL;     
	SurfaceListLen  = 0;
	MaterialList    = NULL;
	MaterialListLen = 0;
	TextureList     = NULL;
	TextureListLen  = 0;
	Visible  = 1;
}


TObject::~TObject(void)
{
}


void TObject::BuildNormals()
{
	int i;
	for(i=0;i<SurfaceListLen;i++)SurfaceList[i].BuildNormals();
}

void TObject::BuildMatrix()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();                          // loads the identity matrix
	glTranslatef  (Pose.x,Pose.y,Pose.z);      // translate the object to the desired position
	glRotatef     (Pose.yaw  ,0.0f,1.0f,0.0f); // rotation around Y axis
	glRotatef     (Pose.pitch,1.0f,0.0f,0.0f); // rotation around X axis
	glRotatef     (Pose.roll ,0.0f,0.0f,1.0f); // rotation around Z axis
	glScalef      (Scale.x,Scale.y,Scale.z);   // Scale object
	glGetFloatv   (GL_MODELVIEW_MATRIX ,TRS.m);//retrieves the TRS Matrix
}

void TObject::Draw()
{
	if(Visible)
	{
		int i;
		for(i=0;i<SurfaceListLen;i++)	
		{
			SurfaceList[i].MaterialList = MaterialList;
			SurfaceList[i].TextureList  = TextureList;
			SurfaceList[i].Draw();
		}
	}
}
void TObject::CreateSampleCube()
{
	DestroyObject();

	MaterialList = new TMaterial[1];
	MaterialListLen = 1;
	MaterialList[0].LoadDefaultMaterial();

	SurfaceList = new TSurface[1];
	SurfaceListLen = 1;
	SurfaceList[0].CreateSurface(8,8,12,0);
	SurfaceList[0].MaterialList = MaterialList;
	
	SurfaceList[0].VertexList[0].vector( 1.0f, 1.0f, 1.0f);
	SurfaceList[0].VertexList[1].vector(-1.0f, 1.0f, 1.0f);
	SurfaceList[0].VertexList[2].vector(-1.0f,-1.0f, 1.0f);
	SurfaceList[0].VertexList[3].vector( 1.0f,-1.0f, 1.0f);
	SurfaceList[0].VertexList[4].vector( 1.0f, 1.0f,-1.0f);
	SurfaceList[0].VertexList[5].vector(-1.0f, 1.0f,-1.0f);
	SurfaceList[0].VertexList[6].vector(-1.0f,-1.0f,-1.0f);
	SurfaceList[0].VertexList[7].vector( 1.0f,-1.0f,-1.0f);

	SurfaceList[0].PolygonList[ 0].Triangle(0,1,2);	
	SurfaceList[0].PolygonList[ 1].Triangle(2,3,0);
	SurfaceList[0].PolygonList[ 2].Triangle(4,7,6);	
	SurfaceList[0].PolygonList[ 3].Triangle(6,5,4);
	SurfaceList[0].PolygonList[ 4].Triangle(4,5,1);	
	SurfaceList[0].PolygonList[ 5].Triangle(1,0,4);
	SurfaceList[0].PolygonList[ 6].Triangle(7,3,2);	
	SurfaceList[0].PolygonList[ 7].Triangle(2,6,7);
	SurfaceList[0].PolygonList[ 8].Triangle(4,0,3);	
	SurfaceList[0].PolygonList[ 9].Triangle(3,7,4);
	SurfaceList[0].PolygonList[10].Triangle(2,1,5);	
	SurfaceList[0].PolygonList[11].Triangle(5,6,2);
	BuildNormals();
}

int TObject::Load3DSObject(char * filename)
{
	strcpy(FileName,filename);
	//SurfaceList[0].MaterialList = MaterialList;
	return 1;
}

void TObject::CreateObject(unsigned int Surfaces, unsigned int Materials)
{
	SurfaceListLen = Surfaces;  SurfaceList = new TSurface[SurfaceListLen];     
	MaterialListLen = Materials; MaterialList = new TMaterial[MaterialListLen];
}

void TObject::DestroyObject()
{
	int i;
	for(i=0;i<SurfaceListLen;i++)	SurfaceList[i].DestroySurface();
	if(SurfaceList) delete [] SurfaceList;     
	SurfaceListLen = 0;    
	if(MaterialList) delete [] MaterialList;
	MaterialListLen = 0;
}


int TObject::LoadOBJObject(char * filename)
{
    DestroyObject();
	char c,c1;
	char Line[1024];

	FILE * fp=fopen(filename,"rt");
	if(fp)
	{
		int a;
		unsigned int Vertices  = 0;
		unsigned int Triangles = 0;
		unsigned int Normals   = 0;
		unsigned int TextCoords = 0;
		strcpy(FileName,filename);
        // stops on the first character of each line
		do{
			do{	c=fgetc(fp); }	while(!feof(fp) && c!='#' && (c<'a' || c>'z'));
			if(!feof(fp))
			{
				 c1=fgetc(fp);
				 switch(c)
				  {
				  case 'v':switch(c1)
						   {
							 case' ':Vertices++;   do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
							 case't':TextCoords++; do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
							 case'n':Normals++;    do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
						   }
						   break;

				  case 'f':Triangles++; do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
				  default: do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n'); break;
				  }
			}
			}while(!feof(fp));
		rewind(fp);//fseek(fp,0,SEEK_SET);
		if(TextCoords == 0) TextCoords = Vertices;
		CreateObject(1,1);
		SurfaceList[0].CreateSurface(Vertices,TextCoords,Triangles,0);
		Vertices   = 0;
		TextCoords = 0;
		Triangles  = 0;
        do{
			do{	c=fgetc(fp); }	while(!feof(fp) && c!='\0' && c!='#' && (c<'a' || c>'z'));
			if(!feof(fp))
			{
				  c1=fgetc(fp);
				  switch(c)
				  {
				  case 'v':switch(c1)
						   {
				             case' ':fscanf(fp,"%f %f %f\n",&SurfaceList[0].VertexList[Vertices].x,
										                    &SurfaceList[0].VertexList[Vertices].y,
															&SurfaceList[0].VertexList[Vertices].z);
								     Vertices++; 
									 break;
							 case't':fscanf(fp,"%f %f\n",&SurfaceList[0].TexCoordList[TextCoords].u,
										                 &SurfaceList[0].TexCoordList[TextCoords].v);//do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
								     TextCoords++;
									 break;
							 case'n':do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
						   }
						   break;

				  case 'f':if(Normals)
						   {
							   if(TextCoords) fscanf(fp,"%d/%d/%d %d/%d/%d %d/%d/%d \n",&SurfaceList[0].PolygonList[Triangles].v0,&SurfaceList[0].PolygonList[Triangles].t0,&a,
								                                                        &SurfaceList[0].PolygonList[Triangles].v1,&SurfaceList[0].PolygonList[Triangles].t1,&a,
																						&SurfaceList[0].PolygonList[Triangles].v2,&SurfaceList[0].PolygonList[Triangles].t2,&a);
							   else           fscanf(fp,"%d//%d %d//%d %d//%d \n"      ,&SurfaceList[0].PolygonList[Triangles].v0,&a,
								                                                        &SurfaceList[0].PolygonList[Triangles].v1,&a,
																						&SurfaceList[0].PolygonList[Triangles].v2,&a);
						   }
						   else
						   {
							   if(TextCoords) fscanf(fp,"%d/%d %d/%d %d/%d \n",&SurfaceList[0].PolygonList[Triangles].v0,&SurfaceList[0].PolygonList[Triangles].t0,
								                                               &SurfaceList[0].PolygonList[Triangles].v1,&SurfaceList[0].PolygonList[Triangles].t1,
																			   &SurfaceList[0].PolygonList[Triangles].v2,&SurfaceList[0].PolygonList[Triangles].t2);
							   else           fscanf(fp,"%d %d %d \n"         ,&SurfaceList[0].PolygonList[Triangles].v0,
								                                               &SurfaceList[0].PolygonList[Triangles].v1,
																			   &SurfaceList[0].PolygonList[Triangles].v2);
						   }
						   Triangles++; break;
				  default: do{c1=fgetc(fp);}while(!feof(fp) && c1!='\n');break;
				  }
			}
			}while(!feof(fp));

		BBox.Min = BBox.Max = SurfaceList[0].VertexList[0];
		for(int i=0;i<SurfaceListLen;i++)
		{
			SurfaceList[i].BBox.Min = SurfaceList[i].BBox.Max = SurfaceList[i].VertexList[0];
			for(int j=1;j<SurfaceList[i].VNListLen;j++)
			{
				if(SurfaceList[i].VertexList[j].x < SurfaceList[i].BBox.Min.x) SurfaceList[i].BBox.Min.x = SurfaceList[i].VertexList[j].x;
				if(SurfaceList[i].VertexList[j].y < SurfaceList[i].BBox.Min.y) SurfaceList[i].BBox.Min.y = SurfaceList[i].VertexList[j].y;
				if(SurfaceList[i].VertexList[j].z < SurfaceList[i].BBox.Min.z) SurfaceList[i].BBox.Min.z = SurfaceList[i].VertexList[j].z;

				if(SurfaceList[i].VertexList[j].x > SurfaceList[i].BBox.Max.x) SurfaceList[i].BBox.Max.x = SurfaceList[i].VertexList[j].x;
				if(SurfaceList[i].VertexList[j].y > SurfaceList[i].BBox.Max.y) SurfaceList[i].BBox.Max.y = SurfaceList[i].VertexList[j].y;
				if(SurfaceList[i].VertexList[j].z > SurfaceList[i].BBox.Max.z) SurfaceList[i].BBox.Max.z = SurfaceList[i].VertexList[j].z;
			}
			    if(SurfaceList[i].BBox.Min.x < BBox.Min.x) BBox.Min.x = SurfaceList[i].BBox.Min.x;
				if(SurfaceList[i].BBox.Min.y < BBox.Min.y) BBox.Min.y = SurfaceList[i].BBox.Min.y;
				if(SurfaceList[i].BBox.Min.z < BBox.Min.z) BBox.Min.z = SurfaceList[i].BBox.Min.z;

				if(SurfaceList[i].BBox.Max.x > BBox.Max.x) BBox.Max.x = SurfaceList[i].BBox.Max.x;
				if(SurfaceList[i].BBox.Max.y > BBox.Max.y) BBox.Max.y = SurfaceList[i].BBox.Max.y;
				if(SurfaceList[i].BBox.Max.z > BBox.Max.z) BBox.Max.z = SurfaceList[i].BBox.Max.z;
		}

		for(int i=0;i<SurfaceList[0].PolygonListLen;i++)
		{
			SurfaceList[0].PolygonList[i].v0--;
			SurfaceList[0].PolygonList[i].v1--;
			SurfaceList[0].PolygonList[i].v2--;
		}
		SurfaceList[0].BuildNormals();
		return 1;
	}
	return 0;
}


int TObject::AddTexture(char * filename, int Interpolation)
{
	TTexture * NewTextureList = new TTexture[TextureListLen+1];
	memcpy(NewTextureList,TextureList,sizeof(TTexture)*TextureListLen);
	if(NewTextureList[TextureListLen].LoadBMPTexture(filename, Interpolation))
	{
		delete [] TextureList;
		TextureList = NewTextureList;
		TextureListLen ++;
		return 1;
	}

	delete [] NewTextureList;
	return 0;
}

int TObject::DeleteTexture(unsigned int Index)
{
	if(Index>=0 && Index<TextureListLen)
	{
		TTexture * NewTextureList = new TTexture[TextureListLen-1];
		if(Index == 0)
		{
			memcpy(NewTextureList,&TextureList[1],sizeof(TTexture)*(TextureListLen-1));
		}
		else
		if(Index == TextureListLen-1)
		{
			memcpy(NewTextureList,TextureList,sizeof(TTexture)*(TextureListLen-1));
		}
		else
		{
			memcpy(NewTextureList,TextureList,sizeof(TTexture)*Index);
			memcpy(&NewTextureList[Index],&TextureList[Index+1],sizeof(TTexture)*(TextureListLen-Index-1));
		}

		glDeleteTextures(1,&TextureList[Index].Texture);
		delete [] TextureList;
		TextureList = NewTextureList;
		//Material Indices correction
		int i,j;
		for(i=0;i<MaterialListLen;i++)
		{
			for(j=0;j<MaterialList[i].TextureLevelListLen;j++)
			{
				if(MaterialList[i].TextureLevelList[j].TextureIndex>Index) MaterialList[i].TextureLevelList[j].TextureIndex--;
				else
				if(MaterialList[i].TextureLevelList[j].TextureIndex==Index) MaterialList[i].RemoveTextureLevel(j);//MaterialList[i].TextureLevelList[j].TextureIndex=-1;
			}
		}
	}
	return TextureListLen;
}