#include "StdAfx.h"
#include "TScene.h"


TScene::TScene(void)
{
	CameraList          =  NULL;;
	CameraListLen       =  0;
	SelectedCamera = -1;
    ObjectList          =  NULL;
	ObjectListLen       =  0;
	Visible        =  1;
	ClearColor.r   =  0.0f;
	ClearColor.g   =  0.0f;
	ClearColor.b   =  0.0f;
	ClearColor.a   =  0.0f;
	LightList          =  NULL;
	LightListLen       =  0;
	EnableBkgTexture   =  0;
}


TScene::~TScene(void)
{
}


void TScene::BuildSceneGeometry()
{
	int i;
	for(i=0;i<ObjectListLen;i++)	ObjectList[i].BuildNormals();
}

void TScene::BuildSceneMatrices()
{
	int i;
	for(i=0;i<CameraListLen;i++) CameraList[i].BuildMatrix();
	for(i=0;i<LightListLen;i++) LightList[i].BuildMatrix();
    for(i=0;i<ObjectListLen;i++) ObjectList[i].BuildMatrix();
}


void DrawAxis()
{
	glColor4f(1.0f,0.0f,0.0f,1.0f);	glBegin(GL_LINES); glVertex3f(-10.0f, 0.0f, 0.0f); glVertex3f(10.0f, 0.0f, 0.0f); glEnd();
	    glColor4f(0.0f,1.0f,0.0f,1.0f);	glBegin(GL_LINES); glVertex3f(0.0f,-10.0f, 0.0f);  glVertex3f(0.0f, 10.0f, 0.0f); glEnd();
	    glColor4f(0.0f,0.0f,1.0f,1.0f);	glBegin(GL_LINES); glVertex3f(0.0f, 0.0f, -10.0f); glVertex3f(0.0f, 0.0f, 10.0f); glEnd();
}

void TScene::DrawBackground()
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0.0f, 1.0f, 0.0f, 1.0f, -1.0f, 1.0f);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_COLOR_MATERIAL);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,Background.Texture);
	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);
	//glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,GL_REPLACE);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(0,0);
		glTexCoord2f(1,0); glVertex2f(1,0);
		glTexCoord2f(1,1); glVertex2f(1,1);
		glTexCoord2f(0,1); glVertex2f(0,1);
	glEnd();
	glDisable(GL_TEXTURE_2D);

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
		
	glEnable(GL_DEPTH_TEST);
	
}

void TScene::Draw()
{
	if(Visible)
	{
		int i;
		// background rendering////////////////////////////////
		if(EnableBkgTexture && Background.Texture) DrawBackground();
		///////////////////////////////////////////////////////
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glLoadMatrixf(CameraList[SelectedCamera].TR.m); //Load the Camera Matrix
		//draw coordinate axes
		glDisable(GL_LIGHTING);
		glEnable (GL_COLOR_MATERIAL);
	    glColor4f(1.0f,0.0f,0.0f,1.0f);	glBegin(GL_LINES); glVertex3f(-10.0f, 0.0f, 0.0f); glVertex3f(10.0f, 0.0f, 0.0f); glEnd();
	    glColor4f(0.0f,1.0f,0.0f,1.0f);	glBegin(GL_LINES); glVertex3f(0.0f,-10.0f, 0.0f);  glVertex3f(0.0f, 10.0f, 0.0f); glEnd();
	    glColor4f(0.0f,0.0f,1.0f,1.0f);	glBegin(GL_LINES); glVertex3f(0.0f, 0.0f, -10.0f); glVertex3f(0.0f, 0.0f, 10.0f); glEnd();
		
		glEnable(GL_LIGHTING);
		glEnable(GL_CULL_FACE);
	    glEnable(GL_DEPTH_TEST);
		glDisable (GL_COLOR_MATERIAL);
		for(i=0;i<ObjectListLen;i++)
		{
			glLoadIdentity();
			glLoadMatrixf(CameraList[SelectedCamera].TR.m); //Load the Camera Matrix
			glMultMatrixf(ObjectList[i].TRS.m);             //Multiplies for the ObjMatrix
			ObjectList[i].Draw();
		}
	}
}


int TScene::LoadScene(char * filename)
{
	FILE * fp = fopen(filename,"rt");
	if(fp)
	{
		int i;
		fscanf(fp,"<Cameras> %d </Cameras>\n",&CameraListLen);
		if(CameraListLen>0) CameraList = new TCamera[CameraListLen];

		for(i=0;i<CameraListLen;i++)	 
			fscanf(fp,"<Cam>%f, %f, %f - %f, %f, %f</Cam>\n",&CameraList[i].Pose.x,
			                                                 &CameraList[i].Pose.y,
															 &CameraList[i].Pose.z,
															 &CameraList[i].Pose.roll,
															 &CameraList[i].Pose.pitch,
															 &CameraList[i].Pose.yaw);

		fscanf(fp,"<Lights> %d </Lights>\n",&LightListLen);
		if(LightListLen>0) LightList = new TLight[LightListLen];

		for(i=0;i<LightListLen;i++)
		{
			fscanf(fp,"<Light>\n<Position>%f, %f, %f</Position>\n",&LightList[i].Position [0],&LightList[i].Position [1],&LightList[i].Position [2]);
			fscanf(fp,"<Index>%d</Index>\n",&LightList[i].Index);
			fscanf(fp,"<Enabled>%d</Enabled>\n",&LightList[i].Enabled);
			fscanf(fp,"<Direction>%f, %f, %f</Direction>\n",&LightList[i].Direction[0],&LightList[i].Direction[1],&LightList[i].Direction[2]);
			fscanf(fp,"<Specular>%f, %f, %f, %f</Specular>\n",&LightList[i].Specular[0],&LightList[i].Specular[1],&LightList[i].Specular[2],&LightList[i].Specular[3]);
			fscanf(fp,"<Diffuse>%f, %f, %f, %f</Diffuse>\n",&LightList[i].Diffuse[0],&LightList[i].Diffuse[1],&LightList[i].Diffuse[2],&LightList[i].Diffuse[3]);
			fscanf(fp,"<Ambient>%f, %f, %f, %f</Ambient>\n",&LightList[i].Ambient[0],&LightList[i].Ambient[1],&LightList[i].Ambient[2],&LightList[i].Ambient[3]);
			fscanf(fp,"<Spot>%f, %f, %f, %f</Spot>\n",&LightList[i].Cutoff,&LightList[i].SpotProfile);
			fscanf(fp,"<Attenuation>%f, %f, %f</Attenuation>\n",&LightList[i].ConstantAttenuation,&LightList[i].LinearAttenuation,&LightList[i].QuadraticAttenuation);
		}
		fclose(fp);
		return 1;
	}
	return 0;

}

int TScene::SaveScene(char * filename)
{
	FILE * fp = fopen(filename,"wt");
	if(fp)
	{
		int i;
		fprintf(fp,"<Cameras> %d </Cameras>\n",CameraListLen);
		for(i=0;i<CameraListLen;i++)	 
			fprintf(fp,"<Cam>%.3f, %.3f, %.3f - %.3f, %.3f, %.3f</Cam>\n",CameraList[i].Pose.x,
			                                                              CameraList[i].Pose.y,
																		  CameraList[i].Pose.z,
																		  CameraList[i].Pose.roll,
																		  CameraList[i].Pose.pitch,
																		  CameraList[i].Pose.yaw);
		fprintf(fp,"<Lights> %d </Lights>\n",LightListLen);

		for(i=0;i<LightListLen;i++)
		{
			fprintf(fp,"<Light>\n<Position>%.3f, %.3f, %.3f</Position>\n",LightList[i].Position [0],LightList[i].Position [1],LightList[i].Position [2]);
			fprintf(fp,"<Index>%d</Index>\n",LightList[i].Index);
			fprintf(fp,"<Enabled>%d</Enabled>\n",LightList[i].Enabled);
			fprintf(fp,"<Direction>%.3f, %.3f, %.3f</Direction>\n",LightList[i].Direction[0],LightList[i].Direction[1],LightList[i].Direction[2]);
			fprintf(fp,"<Specular>%.3f, %.3f, %.3f, %.3f</Specular>\n",LightList[i].Specular[0],LightList[i].Specular[1],LightList[i].Specular[2],LightList[i].Specular[3]);
			fprintf(fp,"<Diffuse>%.3f, %.3f, %.3f, %.3f</Diffuse>\n",LightList[i].Diffuse[0],LightList[i].Diffuse[1],LightList[i].Diffuse[2],LightList[i].Diffuse[3]);
			fprintf(fp,"<Ambient>%.3f, %.3f, %.3f, %.3f</Ambient>\n",LightList[i].Ambient[0],LightList[i].Ambient[1],LightList[i].Ambient[2],LightList[i].Ambient[3]);
			fprintf(fp,"<Spot>%.3f, %.3f, %.3f, %.3f</Spot>\n",LightList[i].Cutoff,LightList[i].SpotProfile);
			fprintf(fp,"<Attenuation>%.3f, %.3f, %.3f</Attenuation>\n",LightList[i].ConstantAttenuation,LightList[i].LinearAttenuation,LightList[i].QuadraticAttenuation);
		}
		fclose(fp);
		return 1;
	}
	return 0;
}

void TScene::CreateScene(unsigned int Cameras,
	                     unsigned int Lights,
	                     unsigned int Objects)
{
	CameraListLen = Cameras;
	CameraList = new TCamera[CameraListLen];
	if(Cameras) SelectedCamera = 0;

	LightList = new TLight[Lights];
	LightListLen = Lights;

	ObjectListLen = Objects;
	ObjectList = new TObject[ObjectListLen];

}

void TScene::DestroyScene()
{
	if(CameraList) {delete [] CameraList;	CameraListLen =  0; SelectedCamera = -1;}
	if(ObjectList) {delete [] ObjectList;	ObjectListLen =  0;}
}


void TScene::SetClearColor(float r, float g, float b, float a)
{
	ClearColor.r = r;
	ClearColor.g = g;
	ClearColor.b = b;
	ClearColor.a = a;
}