#include "StdAfx.h"
#include "TSurface.h"


TSurface::TSurface(void)
{
	VertexList   = NormalList = NULL; 
	TexCoordList = NULL; 
	PolygonList  = NULL;
	VNListLen    = PolygonListLen = 0; 
	TexCoordListLen = 0;
	Material     = -1; 
	Visible      = 1;
}

/*

TSurface::~TSurface(void)
{
}*/
int  TSurface::CreateSurface(unsigned int Vertices, 
	                         unsigned int TexCoords,
	                         unsigned int Polygons, 
							 unsigned int MatIndex)
{
	VertexList   = new TVertex[Vertices];
	NormalList   = new TVector[Vertices];
	VNListLen    = Vertices;
	
	TexCoordList = new TTexCoord[TexCoords];
	TexCoordListLen = TexCoords;

	PolygonList = new TPolygon[Polygons];
	PolygonListLen = Polygons;
	int i;
	Material = MatIndex;
	for(i=0;i<PolygonListLen;i++) 
	{
		PolygonList[i].VertexList   = VertexList;
		PolygonList[i].NormalList   = NormalList;
		PolygonList[i].MaterialList = MaterialList;
		PolygonList[i].Material     = Material;
	}
	
	return 1;
}

int  TSurface::CreateSurface(TVertex * v, 
							 unsigned int Vertices, 
							 TPolygon * p, 
							 unsigned int Polygons, 
							 unsigned int MatIndex)
{
	CreateSurface(Vertices, Vertices, Polygons, MatIndex);
	memcpy(VertexList,v,sizeof(TVertex)*Vertices);
	memcpy(PolygonList,p,sizeof(TVector)*Vertices);
	BuildNormals();
	return 1;
}

void TSurface::DestroySurface()
{
	delete [] VertexList;
	delete [] NormalList;
	delete [] TexCoordList;
	VNListLen = 0;

	delete [] PolygonList;
	PolygonListLen = 0;

	Material = -1;
}

void TSurface::BuildNormals()
{
	int i,j;
	for(i=0;i<PolygonListLen;i++) 
		PolygonList[i].BuildNormal();

	// calculate normalsper vertex
	TVector N;
	N.vector(0,0,0);
	for(i=0;i<VNListLen;i++)
	{
		N.vector(0,0,0);
		for(j=0;j<PolygonListLen;j++)
		{
			if(PolygonList[j].v0==i || PolygonList[j].v1==i || PolygonList[j].v2==i)
			{
				N+=PolygonList[j].Normal;
			}
		}
		N.Normalize();
		NormalList[i] = N;
	}
}

void TSurface::Flip()
{
	int i;
    for(i=0;i<PolygonListLen;i++) PolygonList[i].Flip();
}

void TSurface::Draw()
{
	if(Visible)
	{
		int i,j;
		MaterialList[Material].LoadMaterial();
		if(MaterialList[Material].TextureLevelListLen)
		{
			glEnable(GL_TEXTURE_2D);
			for(j=0;j<MaterialList[Material].TextureLevelListLen;j++)
            {
				if(MaterialList[Material].TextureLevelList[j].Visible)
				{
						glBindTexture(GL_TEXTURE_2D,TextureList[MaterialList[Material].TextureLevelList[j].TextureIndex].Texture);
						glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,MaterialList[Material].TextureLevelList[j].EnvMode);
						//glTexEnvf(GL_TEXTURE_ENV,GL_COMBINE_RGB,MaterialList[Material].TextureLevelList[j].CombineRGB);
						//glTexEnvf(GL_TEXTURE_ENV,GL_COMBINE_ALPHA,MaterialList[Material].TextureLevelList[j].CombineAlpha);
						if(MaterialList[Material].TextureLevelList[j].Blend) 
						{ glEnable(GL_BLEND); glBlendFunc(MaterialList[Material].TextureLevelList[j].BlendSrc, MaterialList[Material].TextureLevelList[j].BlendDst);}

						if(MaterialList[Material].TextureLevelList[j].MappingType == MAPPING_UV)
						{
							for(i=0;i<PolygonListLen;i++)
							{
								PolygonList[i].TexCoordList = TexCoordList;
								PolygonList[i].Material = Material;
								PolygonList[i].MaterialList = MaterialList;
								PolygonList[i].DrawTexture();
							}
						}
						else
						if(MaterialList[Material].TextureLevelList[j].MappingType == MAPPING_SPHERE)
						{
							glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
							glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
							glEnable(GL_TEXTURE_GEN_T);
							glEnable(GL_TEXTURE_GEN_S);	
							for(i=0;i<PolygonListLen;i++)
							{
								PolygonList[i].Material = Material;
								PolygonList[i].MaterialList = MaterialList;
								PolygonList[i].Draw();
							}
							glDisable(GL_TEXTURE_GEN_S); 
							glDisable(GL_TEXTURE_GEN_T);
						}
						glDisable(GL_BLEND);
				}
			}
			glDisable(GL_TEXTURE_2D);
		}
		else
		{
			for(i=0;i<PolygonListLen;i++)
			{
				PolygonList[i].Material = Material;
				PolygonList[i].MaterialList = MaterialList;
				PolygonList[i].Draw();
			}
		}
		/*glEnable (GL_COLOR_MATERIAL);
		glColor4f(1.0f,1.0f,0.0f,1.0f);
		for(i=0;i<VNListLen;i++)
		{
			glBegin(GL_LINES);
			 glVertex3f(VertexList[i].x,VertexList[i].y,VertexList[i].z);
			 glVertex3f(VertexList[i].x+NormalList[i].x,VertexList[i].y+NormalList[i].y,VertexList[i].z+NormalList[i].z);
			glEnd();
		}
		/*for(i=0;i<PolygonListLen;i++)
		{
			PolygonList[i].DrawNormal();
		}*/
	}
}